@@ -22,6 +22,7 @@ require 'rbox_edit_usb.pl';
 require 'rbox_edit_shared.pl';
 our (%gui, %vmc);
 
+# Show the Edit dialog for a Guest
 sub show_dialog_edit {
     undef(%vmc);
     my $gref = &getsel_list_guest();
@@ -35,7 +36,8 @@ sub show_dialog_edit {
         $vmc{USBFilters} = IMachine_getUSBDeviceFilters($vmc{IMachine});
         $vmc{IBIOSSettings} = IMachine_getBIOSSettings($vmc{IMachine});
         $vmc{IGraphicsAdapter} = IMachine_getGraphicsAdapter($vmc{IMachine});
-        $vmc{IAudioAdapter} = IMachine_getAudioAdapter($vmc{IMachine});
+        $vmc{IAudioSettings} = IMachine_getAudioSettings($vmc{IMachine});
+        $vmc{IAudioAdapter} = IAudioSettings_getAdapter($vmc{IAudioSettings});
         $vmc{IVRDEServer} = IMachine_getVRDEServer($vmc{IMachine});
         $vmc{IParallelPort} = IMachine_getParallelPort($vmc{IMachine}, 0);
         $vmc{IRecordingSettings} = IMachine_getRecordingSettings($vmc{IMachine});
@@ -81,6 +83,7 @@ sub show_dialog_edit {
             $encryptionpage->set_sensitive(1);
             $gui{hscaleEditSysProcessor}->set_sensitive(1);
             $gui{checkbuttonEditSysPAE}->set_sensitive(1);
+            $gui{checkbuttonEditSysNestedVTX}->set_sensitive(1);
             $gui{checkbuttonEditSysCPUHotPlug}->set_sensitive(1);
             $dispvideopage->set_sensitive(1);
             $gui{checkbuttonEditDispMultiple}->set_sensitive(1);
@@ -110,6 +113,7 @@ sub show_dialog_edit {
             $encryptionpage->set_sensitive(0);
             $gui{hscaleEditSysProcessor}->set_sensitive(0);
             $gui{checkbuttonEditSysPAE}->set_sensitive(0);
+            $gui{checkbuttonEditSysNestedVTX}->set_sensitive(0);
             $gui{checkbuttonEditSysCPUHotPlug}->set_sensitive(0);
             $dispvideopage->set_sensitive(0);
             $gui{checkbuttonEditDispMultiple}->set_sensitive(0);
@@ -185,7 +189,7 @@ sub show_dialog_edit {
 
         undef(%vmc);
         &flush_pagecache();
-        $gui{menuitemAttachFloppy} = $gui{menuitemAttachDVD} = $gui{menuitemAttachHD} = $gui{menuAttachFloppy} =
+        $gui{miAttachFloppy} = $gui{miAttachDVD} = $gui{miAttachHD} = $gui{menuAttachFloppy} =
         $gui{menuAttachDVD} = $gui{menuAttachHD} = $gui{menuAttachAdd} = $gui{menuUSB} = undef; # These must be freed
     }
     else { &show_err_msg('settings'); }
