@@ -7,9 +7,9 @@ our (%gui, %signal, %vmc);
 sub init_edit_audio {
     &set_pointer($gui{dialogEdit}, 'watch');
     my $vhost = &vhost();
-    $gui{comboboxEditAudioDriver}->signal_handler_block($signal{audiodrv});
-    $gui{comboboxEditAudioCtr}->signal_handler_block($signal{audioctr});
-    $gui{comboboxEditAudioCodec}->signal_handler_block($signal{audiocodec});
+    $gui{comboboxEditAudioDriver}->signal_handler_block($signal{comboboxEditAudioDriver_changed});
+    $gui{comboboxEditAudioCtr}->signal_handler_block($signal{comboboxEditAudioCtr_changed});
+    $gui{comboboxEditAudioCodec}->signal_handler_block($signal{comboboxEditAudioCodec_changed});
     $gui{checkbuttonEditAudioEnable}->set_active(&bl(IAudioAdapter_getEnabled($vmc{IAudioAdapter})));
 
     # Ghost / Unghost other widgets based on online editing and whether audio is enabled or not.
@@ -40,9 +40,9 @@ sub init_edit_audio {
     &combobox_set_active_text($gui{comboboxEditAudioCodec}, IAudioAdapter_getAudioCodec($vmc{IAudioAdapter}), 0);
     &combobox_set_active_text($gui{comboboxEditAudioDriver}, IAudioAdapter_getAudioDriver($vmc{IAudioAdapter}), 0);
     &combobox_set_active_text($gui{comboboxEditAudioCtr}, $controller, 0);
-    $gui{comboboxEditAudioDriver}->signal_handler_unblock($signal{audiodrv});
-    $gui{comboboxEditAudioCtr}->signal_handler_unblock($signal{audioctr});
-    $gui{comboboxEditAudioCodec}->signal_handler_unblock($signal{audiocodec});
+    $gui{comboboxEditAudioDriver}->signal_handler_unblock($signal{comboboxEditAudioDriver_changed});
+    $gui{comboboxEditAudioCtr}->signal_handler_unblock($signal{comboboxEditAudioCtr_changed});
+    $gui{comboboxEditAudioCodec}->signal_handler_unblock($signal{comboboxEditAudioCodec_changed});
     &set_pointer($gui{dialogEdit});
 }
 
