@@ -8,8 +8,8 @@ sub init_edit_gen_basic {
     my ($osfam, $osver) = &osfamver();
     my $Name = IMachine_getName($vmc{IMachine});
     my $Osid = IMachine_getOSTypeId($vmc{IMachine});
-    $gui{comboboxEditGenOSFam}->signal_handler_block($signal{famedit});
-    $gui{comboboxEditGenOSVer}->signal_handler_block($signal{veredit});
+    $gui{comboboxEditGenOSFam}->signal_handler_block($signal{comboboxEditGenOSFam_changed});
+    $gui{comboboxEditGenOSVer}->signal_handler_block($signal{comboboxEditGenOSVer_changed});
     $gui{liststoreEditGenOSFam}->clear();
     $gui{liststoreEditGenOSVer}->clear();
     my $IGuestOSType = IVirtualBox_getGuestOSType($gui{websn}, $Osid);
@@ -31,8 +31,8 @@ sub init_edit_gen_basic {
         $gui{comboboxEditGenOSVer}->set_active_iter($iter) if ($_ eq $$IGuestOSType{id});
     }
 
-    $gui{comboboxEditGenOSFam}->signal_handler_unblock($signal{famedit});
-    $gui{comboboxEditGenOSVer}->signal_handler_unblock($signal{veredit});
+    $gui{comboboxEditGenOSFam}->signal_handler_unblock($signal{comboboxEditGenOSFam_changed});
+    $gui{comboboxEditGenOSVer}->signal_handler_unblock($signal{comboboxEditGenOSVer_changed});
     &set_pointer($gui{dialogEdit});
 }
 
@@ -49,8 +49,8 @@ sub gen_basic_os_family {
     my ($combofam, $combover) = @_;
     my ($osfam, $osver) = &osfamver();
     my $fam = &getsel_combo($combofam, 1);
-    $combofam->signal_handler_block($signal{famedit}); # Block to avoid signal emission when changing
-    $combover->signal_handler_block($signal{veredit});
+    $combofam->signal_handler_block($signal{comboboxEditGenOSFam_changed}); # Block to avoid signal emission when changing
+    $combover->signal_handler_block($signal{comboboxEditGenOSVer_changed});
     $gui{liststoreEditGenOSVer}->clear();
 
     foreach (sort {
@@ -65,8 +65,8 @@ sub gen_basic_os_family {
     }
 
     $combover->set_active(0) if ($combover->get_active() == -1);
-    $combofam->signal_handler_unblock($signal{famedit});
-    $combover->signal_handler_unblock($signal{veredit});
+    $combofam->signal_handler_unblock($signal{comboboxEditGenOSFam_changed});
+    $combover->signal_handler_unblock($signal{comboboxEditGenOSVer_changed});
     $combover->signal_emit('changed'); # Force update of other fields based on OS
 }
 
