@@ -15,7 +15,7 @@ sub init_edit_shared {
         $gui{checkbuttonSharedFolderPermanent}->show();
     }
 
-    &fill_list_editshared($vmc{IMachine});
+    &fill_list_edit_shared($vmc{IMachine});
     IMachine_saveSettings($vmc{IMachine});
     &set_pointer($gui{dialogEdit});
 }
@@ -59,7 +59,7 @@ sub show_dialog_shared {
                                                            $gui{checkbuttonSharedFolderMount}->get_active()) if ($IConsole);
                 }
 
-                &fill_list_editshared($vmc{IMachine});
+                &fill_list_edit_shared($vmc{IMachine});
                 IMachine_saveSettings($vmc{IMachine});
             }
         }
@@ -79,7 +79,7 @@ sub share_remove {
         IConsole_removeSharedFolder($IConsole, $$shareref{Name}) if ($IConsole);
     }
 
-    &fill_list_editshared($vmc{IMachine});
+    &fill_list_edit_shared($vmc{IMachine});
     IMachine_saveSettings($vmc{IMachine});
 }
 
