@@ -39,7 +39,7 @@ sub sys_proc_hotplug {
 # Set the number of processors. The exact mechanism varies depending on hot plug mode
 sub sys_proc_count {
     if ($vmc{SessionType} eq 'WriteLock') {
-        my $newcpucount = int($gui{adjustmentEditSysProcessor}->get_value());
+        my $newcpucount = int($gui{adjEditSysProcessor}->get_value());
 
         # In hot-plug mode, if we reduce the number of CPUs, we must offline them first.
         if (&bl(IMachine_getCPUHotPlugEnabled($vmc{IMachine}))) {
@@ -53,31 +53,19 @@ sub sys_proc_count {
         }
 
         IMachine_setCPUCount($vmc{IMachine}, $newcpucount);
-        return 0;
     }
 }
 
-# Formats the slider value for the processors
-sub sys_proc_count_format {
-    my ($widget, $value) = @_;
-    return " $value vCPUs";
-}
-
 # Sets the processor execution cap
 sub sys_proc_cap {
-    IMachine_setCPUExecutionCap($vmc{IMachine}, int($gui{adjustmentEditSysProcessorCap}->get_value()));
-    return 0;
+    IMachine_setCPUExecutionCap($vmc{IMachine}, int($gui{adjEditSysProcessorCap}->get_value()));
 }
 
-# Formats the slider for the execution cap
-sub sys_proc_cap_format {
-    my ($widget, $value) = @_;
-    return " $value% Max";
-}
 
 # Whether the guest uses Physical Address Extensions and No Execution
 sub sys_proc_pae { if ($vmc{SessionType} eq 'WriteLock') { IMachine_setCPUProperty($vmc{IMachine}, 'PAE', $gui{checkbuttonEditSysPAE}->get_active()); } }
 
 # Whether Nested Hardware Virtualization is enabled on processors which support it
 sub sys_proc_nested_vtx { if ($vmc{SessionType} eq 'WriteLock') { IMachine_setCPUProperty($vmc{IMachine}, 'HWVirt', $gui{checkbuttonEditSysNestedVTX}->get_active()); } }
+
 1;
