@@ -11,7 +11,7 @@ sub init_edit_usb {
     # O=0 E=0 X=1 : USB Enabled. Version 3.0
     &set_pointer($gui{dialogEdit}, 'watch');
     my @IUSBController = IMachine_getUSBControllers($vmc{IMachine});
-    $gui{checkbuttonEditUSBEnable}->signal_handler_block($signal{usbtoggle});
+    $gui{checkbuttonEditUSBEnable}->signal_handler_block($signal{checkbuttonEditUSBEnable_toggled});
     my $usbstate = 0;
 
     foreach my $usbcontroller (@IUSBController) {
@@ -38,7 +38,7 @@ sub init_edit_usb {
         $item->signal_connect(activate => \&usb_add_filter, $usbid);
     }
 
-    $gui{checkbuttonEditUSBEnable}->signal_handler_unblock($signal{usbtoggle});
+    $gui{checkbuttonEditUSBEnable}->signal_handler_unblock($signal{checkbuttonEditUSBEnable_toggled});
     &set_pointer($gui{dialogEdit});
 }
 
@@ -202,20 +202,14 @@ sub usb_move_filter {
 
 # Can't use getsel_list_usbfilters in here due to the way the signals are propagated
 sub usb_toggle_filter {
-    my ($widget, $path_str, $model) = @_;
-    my $iter = $model->get_iter(Gtk3::TreePath->new_from_string($path_str));
-    my $val = $model->get($iter, 0);
-    my $IUSBDeviceFilter = $model->get($iter, 1);
+    my ($widget, $path_str) = @_;
+    my $iter = $gui{liststoreEditUSBFilter}->get_iter(Gtk3::TreePath->new_from_string($path_str));
+    my $val = $gui{liststoreEditUSBFilter}->get($iter, 0);
+    my $IUSBDeviceFilter = $gui{liststoreEditUSBFilter}->get($iter, 1);
     IUSBDeviceFilter_setActive($IUSBDeviceFilter, !$val); # Always set to the opposite
     &fill_list_usbfilters($vmc{IMachine});
 }
 
-sub usb_show_menu {
-    my ($widget, $event) = @_;
-    $gui{menuUSB}->popup(undef, undef, undef, undef, 0, $event->time) if ($event->button == 1);
-    return 0;
-}
-
 # Sets the sensitivity depending on whether USB is enabled or not
 sub usb_sens_enabled {
     my ($state) = @_;
