@@ -1,6 +1,8 @@
 # Error Handling
 use strict;
 use warnings;
+use utf8;
+
 our (%gui);
 
 # Error Dialog Block
@@ -36,7 +38,7 @@ our (%gui);
                                        ' ',
                                        'error'],
                 applimport         => ['Import Appliance Failed',
-                                       'The appliance failed to import correctly. The appliance may be corrupt or in an unsupported format',
+                                       'The appliance failed to import. Ensure the appliance is accessible and a supported format.',
                                        'error'],
                 createguest        => ['Failed to Create Guest',
                                        'Verify your guest settings. Cause may be a duplicate guest name.',
@@ -48,7 +50,7 @@ our (%gui);
                                        'The controller still has media attached. Remove media before removing controller.',
                                        'warning'],
                 ctrfull            => ['Controller Full',
-                                       'This controller is full, no more media can be attached.',
+                                       'This controller is full, no more devices or media can be attached.',
                                        'warning'],
                 diskimgcreation    => ['Failed to Create Disk Image',
                                        'VirtualBox could not create the disk image. You may not have permission or the destination may ' .
@@ -66,6 +68,12 @@ our (%gui);
                 mediuminuse        => ['Medium In Use',
                                        'This medium could not be released from at least one guest as it may be in use',
                                        'warning'],
+                nofloppydrive      => ['No Floppy Drive Found',
+                                       'Could not change disk. This guest does not have a usable floppy drive attached.',
+                                       'warning'],
+                noopticaldrive     => ['No Optical Drive Found',
+                                       'Could not change disc. This guest does not have a usable optical drive attached.',
+                                       'warning'],
                 noextensions       => ['VirtualBox Extension Pack',
                                        'This server does not have the Oracle VirtualBox Extension Pack or VNC Extension Pack installed. ' .
                                        'You should install the Oracle Extension Pack if available for your OS, otherwise ' .
@@ -104,6 +112,12 @@ our (%gui);
                 snapdelchild       => ['Snapshot Deletion Failed',
                                        'Snapshot has 2 or more children. You must delete the child snapshots first.',
                                        'error'],
+                nouefivarfile      => ['No UEFI Variables File',
+                                       'RemoteBox will now initialise the UEFI variables file.',
+                                       'warning'],
+                resetbootkeys      => ['Not Supported Yet',
+                                       'RemoteBox does not yet support resetting secure boot keys',
+                                       'warning'],
                 nohdresize         => ['Virtual Hard Disk Resize Failed',
                                        'The image format, variant or state does not support resizing',
                                        'warning'],
@@ -122,7 +136,7 @@ our (%gui);
                                        'This version of RemoteBox is not supported with the version of VirtualBox on the' .
                                        " server.\nVisit http://remotebox.knobgoblin.org.uk to download a compatible version of RemoteBox.\n" .
                                        "RemoteBox will continue, but you may experience problems due to incompatibilities.\n\n" .
-                                       'Supported VirtualBox Version: 6.1.x',
+                                       'Supported VirtualBox Version: 7.0.x',
                                        'warning'],
                 webservice         => ['VirtualBox Returned an Error',
                                        '',
