@@ -29,6 +29,7 @@ our (%gui, $sharedir);
                   minhdsizemb       => 4, # Not returnable by API but VB itself uses this value
                   maxhdsize         => ISystemProperties_getInfoVDSize($ISystemProperties),
                   maxnet            => ISystemProperties_getMaxNetworkAdapters($ISystemProperties, 'PIIX3'),
+                  hwexclusive       => ISystemProperties_getExclusiveHwVirt($ISystemProperties),
                   maxser            => ISystemProperties_getSerialPortCount($ISystemProperties),
                   minguestcpu       => ISystemProperties_getMinGuestCPUCount($ISystemProperties),
                   maxguestcpu       => ISystemProperties_getMaxGuestCPUCount($ISystemProperties),
@@ -42,7 +43,6 @@ our (%gui, $sharedir);
                   vrdeextpack       => ISystemProperties_getDefaultVRDEExtPack($ISystemProperties),
                   vrdelib           => ISystemProperties_getVRDEAuthLibrary($ISystemProperties),
                   additionsiso      => ISystemProperties_getDefaultAdditionsISO($ISystemProperties),
-                  hwexclusive       => ISystemProperties_getExclusiveHwVirt($ISystemProperties),
                   autostartdb       => ISystemProperties_getAutostartDatabasePath($ISystemProperties));
 
         # Convenience Keys
@@ -58,6 +58,9 @@ our (%gui, $sharedir);
         # Ontain any physical floppy drives
         my @floppy = IHost_getFloppyDrives($IHost);
         $vhost{floppy} = \@floppy;
+
+        my @tpm = ISystemProperties_getSupportedTpmTypes($ISystemProperties);
+        $vhost{tpmtypes} = \@tpm;
     }
 }
 
@@ -117,8 +120,8 @@ our (%gui, $sharedir);
             $osver{$$type{id}}{recommendedAudioController} = $$type{recommendedAudioController};
             $osver{$$type{id}}{familyId} = $$type{familyId};
             if (-e "$sharedir/icons/os/$$type{id}.png") { $osver{$$type{id}}{icon} = Gtk3::Gdk::Pixbuf->new_from_file("$sharedir/icons/os/$$type{id}.png"); }
-            elsif ($$type{id} =~ m/_64$/) { $osver{$$type{id}}{icon} = $gui{img}{OtherOS64}; }
-            else { $osver{$$type{id}}{icon} = $gui{img}{OtherOS}; }
+            elsif ($$type{id} =~ m/_64$/) { $osver{$$type{id}}{icon} = $gui{pb}{OtherOS64}; }
+            else { $osver{$$type{id}}{icon} = $gui{pb}{OtherOS}; }
         }
     }
 }
@@ -343,13 +346,14 @@ sub random_key {
 # Simple XOR with password and key
 sub xor_pass {
     my ($pass, $key) = @_;
-
     my $encpass = '';
+
     foreach my $char (split //, $pass) {
         my $decode = chop($key);
         $encpass .= chr(ord($char) ^ ord($decode));
         $key = $decode . $key;
     }
+
     return $encpass;
 }
 
