@@ -1,6 +1,8 @@
 # Main Entry for RemoteBox
 use strict;
 use warnings;
+use Data::Dumper;
+
 our (%gui, %prefs);
 my %cmdopts=();
 $endpoint = 'http://localhost:18083';
@@ -31,14 +33,6 @@ Gtk3->main;
 sub quit_remotebox {
     &save_window_pos('windowMain'); # Also saves the main preferences
     &virtualbox_logoff();
-
-    # These should be reset on exit to help garbage collection
-    $gui{menuitemFloppy}->set_submenu(undef);
-    $gui{menuitemDVD}->set_submenu(undef);
-    $gui{menuitemUSB}->set_submenu(undef);
-    $gui{menuitemFloppy}->set_submenu($gui{menutmp1});
-    $gui{menuitemDVD}->set_submenu($gui{menutmp2});
-    $gui{menuitemUSB}->set_submenu($gui{menutmp3});
     Gtk3->main_quit;
 }
 
@@ -164,7 +158,7 @@ sub show_dialog_connect {
             }
             else {
                 &addrow_msg_log("Logged into $endpoint", "Running VirtualBox $ver");
-                &show_err_msg('vboxver', "Detected VirtualBox Version: $ver") if ($ver !~ m/^6.1/);
+                &show_err_msg('vboxver', "Detected VirtualBox Version: $ver") if ($ver !~ m/^7.0/);
                 my $vhost = &vhost();
 
                 if ($$vhost{vrdeextpack} =~ m/Oracle VM VirtualBox Extension Pack/i) { $gui{toolbuttonRemoteDisplay}->set_label('_Guest Display (RDP)'); }
@@ -178,9 +172,11 @@ sub show_dialog_connect {
                 }
 
                 if ($prefs{ADDADDITIONS}) {
-                    &addrow_msg_log('Adding Guest Additions ISO to VMM');
-                    my $additions = IVirtualBox_openMedium($gui{websn}, $$vhost{additionsiso}, 'DVD', 'ReadOnly', 'false') if ($$vhost{additionsiso});
-                    if (!defined($additions)) { &addrow_msg_log('Warning: Could not automatically add guest additions ISO to VMM.'); }
+                    if ($$vhost{additionsiso}) {
+                        my ($vol, $dir, $file) = &rsplitpath($$vhost{additionsiso});
+                        my $IMedium = &register_medium(undef, {type => 'DVD', mode => 'ReadOnly', basedir => $vol . $dir, file => $file});
+                        &addrow_msg_log('Warning: Could not automatically register the guest additions ISO with the VMM') if (!defined($IMedium));
+                    }
                 }
 
                 &fill_list_guest();
@@ -279,7 +275,7 @@ sub show_dialog_exportappl {
     my $fname = &rcatdir($$vhost{machinedir}, $$gref{Name});
     $gui{entryExportApplFile}->set_text($fname);
     $gui{entryExportApplName}->set_text($$gref{Name});
-    $gui{textbufferExportApplDescription}->set_text(IMachine_getDescription($$gref{IMachine}));
+    $gui{txtbExportApplDescription}->set_text(IMachine_getDescription($$gref{IMachine}));
 
     # Determine if any media are USB attached as they are not supported by Appliances and
     # whether any media encrypted
@@ -324,10 +320,10 @@ sub show_dialog_exportappl {
                 IVirtualSystemDescription_addDescription($IVirtualSystemDescription, 'Vendor', $gui{entryExportApplVendor}->get_text(), '');
                 IVirtualSystemDescription_addDescription($IVirtualSystemDescription, 'VendorUrl', $gui{entryExportApplVendorURL}->get_text(), '');
                 IVirtualSystemDescription_addDescription($IVirtualSystemDescription, 'Version', $gui{entryExportApplVersion}->get_text(), '');
-                my ($iter_s, $iter_e) = ($gui{textbufferExportApplDescription}->get_start_iter(), $gui{textbufferExportApplDescription}->get_end_iter());
-                IVirtualSystemDescription_addDescription($IVirtualSystemDescription, 'Description', $gui{textbufferExportApplDescription}->get_text($iter_s, $iter_e, 0), '');
-                ($iter_s, $iter_e) = ($gui{textbufferExportApplLicense}->get_start_iter(), $gui{textbufferExportApplLicense}->get_end_iter());
-                IVirtualSystemDescription_addDescription($IVirtualSystemDescription, 'License', $gui{textbufferExportApplLicense}->get_text($iter_s, $iter_e, 0), '');
+                my ($iter_s, $iter_e) = ($gui{txtbExportApplDescription}->get_start_iter(), $gui{txtbExportApplDescription}->get_end_iter());
+                IVirtualSystemDescription_addDescription($IVirtualSystemDescription, 'Description', $gui{txtbExportApplDescription}->get_text($iter_s, $iter_e, 0), '');
+                ($iter_s, $iter_e) = ($gui{txtbExportApplLicense}->get_start_iter(), $gui{txtbExportApplLicense}->get_end_iter());
+                IVirtualSystemDescription_addDescription($IVirtualSystemDescription, 'License', $gui{txtbExportApplLicense}->get_text($iter_s, $iter_e, 0), '');
                 # The export options array is not at the end of the function so we have to make it a fixed size. In this case 3 elements.
                 # We also cannot pass a null option, so choose something harmless and duplicate (CreateManifest). Messy but no way around it...
                 my @exportopts = ('CreateManifest', 'CreateManifest', 'CreateManifest');
@@ -335,7 +331,7 @@ sub show_dialog_exportappl {
                 $exportopts[2] = 'StripAllNonNATMACs' if ($gui{comboboxExportApplMAC}->get_active() == 1);
                 $exportopts[2] = 'StripAllMACs' if ($gui{comboboxExportApplMAC}->get_active() == 2);
                 my $IProgress = IAppliance_write($IAppliance, &getsel_combo($gui{comboboxExportApplFormat}, 1), @exportopts, $location);
-                if ($IProgress) { &show_progress_window($IProgress, "Exporting Appliance $$gref{Name}", $gui{img}{ProgressExport}); }
+                if ($IProgress) { &show_progress_window($IProgress, "Exporting Appliance $$gref{Name}", $gui{pb}{ProgressExport}); }
                 &addrow_msg_log("Exported $$gref{Name} as an appliance to $location");
             }
         }
@@ -344,18 +340,18 @@ sub show_dialog_exportappl {
     } until (!$gui{dialogExportAppl}->get_visible());
 }
 
-sub show_dialog_snapshotdetails {
+sub show_dialog_snap_details {
     my $snapref = &getsel_list_snapshots();
 
     if ($$snapref{ISnapshot}) { # This is needed in case a user double clicks on "Current State" which is not a real snapshot
         $gui{entrySnapshotName}->set_text(ISnapshot_getName($$snapref{ISnapshot}));
-        $gui{textbufferSnapshotDescription}->set_text(ISnapshot_getDescription($$snapref{ISnapshot}));
+        $gui{txtbSnapshotDescription}->set_text(ISnapshot_getDescription($$snapref{ISnapshot}));
         my $response = $gui{dialogSnapshot}->run;
         $gui{dialogSnapshot}->hide;
 
         if ($response eq 'ok') {
-            my ($iter_s, $iter_e) = ($gui{textbufferSnapshotDescription}->get_start_iter(), $gui{textbufferSnapshotDescription}->get_end_iter());
-            ISnapshot_setDescription($$snapref{ISnapshot}, $gui{textbufferSnapshotDescription}->get_text($iter_s, $iter_e, 0));
+            my ($iter_s, $iter_e) = ($gui{txtbSnapshotDescription}->get_start_iter(), $gui{txtbSnapshotDescription}->get_end_iter());
+            ISnapshot_setDescription($$snapref{ISnapshot}, $gui{txtbSnapshotDescription}->get_text($iter_s, $iter_e, 0));
             ISnapshot_setName($$snapref{ISnapshot}, $gui{entrySnapshotName}->get_text());
             &fill_list_guest();
         }
@@ -364,13 +360,13 @@ sub show_dialog_snapshotdetails {
 
 sub show_dialog_snapshot {
     $gui{entrySnapshotName}->set_text('Snapshot');
-    $gui{textbufferSnapshotDescription}->set_text('');
+    $gui{txtbSnapshotDescription}->set_text('');
     my $response = $gui{dialogSnapshot}->run();
     $gui{dialogSnapshot}->hide();
 
     if ($response eq 'ok') {
-        my ($iter_s, $iter_e) = ($gui{textbufferSnapshotDescription}->get_start_iter(), $gui{textbufferSnapshotDescription}->get_end_iter());
-        &take_snapshot($gui{entrySnapshotName}->get_text(), $gui{textbufferSnapshotDescription}->get_text($iter_s, $iter_e, 0));
+        my ($iter_s, $iter_e) = ($gui{txtbSnapshotDescription}->get_start_iter(), $gui{txtbSnapshotDescription}->get_end_iter());
+        &take_snapshot($gui{entrySnapshotName}->get_text(), $gui{txtbSnapshotDescription}->get_text($iter_s, $iter_e, 0));
     }
 }
 
@@ -413,7 +409,7 @@ sub stop_guest_poweroff {
         my $IConsole = ISession_getConsole($$sref{ISession});
 
         if (my $IProgress = IConsole_powerDown($IConsole)) { # Not cancellable
-            &show_progress_window($IProgress, "Powering off $$gref{Name}", $gui{img}{ProgressPowerOff});
+            &show_progress_window($IProgress, "Powering off $$gref{Name}", $gui{pb}{ProgressPowerOff});
             &addrow_msg_log("Power off signal sent to $$gref{Name}");
             &fill_list_guest();
         }
@@ -444,7 +440,7 @@ sub stop_guest_savestate {
 
     if ($$sref{Lock} eq 'Shared') {
         if (my $IProgress = IMachine_saveState($$sref{IMachine})) { # Not cancellable
-            &show_progress_window($IProgress, "Saving execution state of $$gref{Name}", $gui{img}{ProgressStateSave});
+            &show_progress_window($IProgress, "Saving execution state of $$gref{Name}", $gui{pb}{ProgressStateSave});
             &addrow_msg_log("Execution state of $$gref{Name} saved");
             &fill_list_guest();
         }
@@ -493,7 +489,7 @@ sub start_guest {
     my $started = 0;
 
     if ($IProgress) { # Is Cancellable
-        my $resultcode = &show_progress_window($IProgress, "Starting guest $$gref{Name}", $gui{img}{ProgressStart});
+        my $resultcode = &show_progress_window($IProgress, "Starting guest $$gref{Name}", $gui{pb}{ProgressStart});
 
         if (IProgress_getCanceled($IProgress) eq 'true') { &addrow_msg_log("tarting guest $$gref{Name} cancelled"); }
         elsif ($resultcode != 0) {
@@ -560,7 +556,7 @@ sub remove_guest {
 
         foreach my $medium (@IMedium) { # Is cancellable
             my $IProgress = IMachine_deleteConfig($$gref{IMachine}, $medium);
-            &show_progress_window($IProgress, 'Deleting hard disk image', $gui{img}{ProgressMediaDelete});
+            &show_progress_window($IProgress, 'Deleting hard disk image', $gui{pb}{ProgressMediaDelete});
             if (IProgress_getCanceled($IProgress) eq 'true') { &addrow_msg_log('Cancelled hard disk deletion'); }
         }
 
@@ -578,7 +574,7 @@ sub restore_snapshot {
     if ($$sref{Type} eq 'WriteLock') { # Not cancellable
         my $snapref = &getsel_list_snapshots();
         my $IProgress = IMachine_restoreSnapshot($$sref{IMachine}, $$snapref{ISnapshot});
-        &show_progress_window($IProgress, 'Restoring snapshot', $gui{img}{ProgressSnapshotRestore});
+        &show_progress_window($IProgress, 'Restoring snapshot', $gui{pb}{ProgressSnapshotRestore});
         &addrow_msg_log("Restored snapshot of $$gref{Name}");
     }
     else { &show_err_msg('restorefail', " ($$gref{Name})"); }
@@ -599,7 +595,7 @@ sub delete_snapshot {
         else { # Not cancellable
             my $snapuuid = ISnapshot_getId($$snapref{ISnapshot});
             my $IProgress = IMachine_deleteSnapshot($$sref{IMachine}, $snapuuid);
-            &show_progress_window($IProgress, 'Deleting snapshot', $gui{img}{ProgressSnapshotDiscard}) if ($IProgress);
+            &show_progress_window($IProgress, 'Deleting snapshot', $gui{pb}{ProgressSnapshotDiscard}) if ($IProgress);
             &addrow_msg_log("Deleted snapshot of $$gref{Name}");
         }
     }
@@ -618,7 +614,7 @@ sub take_snapshot {
 
     if ($$sref{Lock} ne 'None') { # Not cancellable
         my ($snapid, $IProgress) = IMachine_takeSnapshot($$sref{IMachine}, $name, $description, 'true');
-        &show_progress_window($IProgress, 'Taking snapshot', $gui{img}{ProgressSnapshotCreate}) if ($IProgress);
+        &show_progress_window($IProgress, 'Taking snapshot', $gui{pb}{ProgressSnapshotCreate}) if ($IProgress);
         &addrow_msg_log("Created snapshot of $$gref{Name} called $name");
     }
     else { &show_err_msg('snapshotfail', " ($$gref{Name})"); }
@@ -691,7 +687,7 @@ sub show_rmb_menu {
             $gui{treeviewGuest}->set_cursor($path, undef, 0);
         }
 
-        $gui{menuMachine}->popup(undef, undef, undef, undef, 0, $event->time);
+        $gui{menuDevices}->popup(undef, undef, undef, undef, 0, $event->time);
         return 1;
     }
 
@@ -699,38 +695,33 @@ sub show_rmb_menu {
 }
 
 # Called when parent DVD menu is highlighted. Cheaper than calling each time main menu is opened
-sub fill_menu_dvd {
+sub fill_menu_optical {
     my $vhost = &vhost();
-    my $dvdmenu = Gtk3::Menu->new();
-    $gui{menuitemDVD}->set_submenu(undef); # Help garbage collection
-    $gui{menuitemDVD}->set_submenu($dvdmenu); # Hijack the temporary submenu (restored on exit)
-    my $IMediumRef = &get_all_media('DVD');
-    my $item = Gtk3::MenuItem->new_with_label('<Empty Drive>');
-    $dvdmenu->append($item);
-    $item->show();
-    $item->signal_connect(activate => \&mount_dvd_online, '');
-    my $sep = Gtk3::SeparatorMenuItem->new();
-    $dvdmenu->append($sep);
-    $sep->show();
+    my $menu = &gui_new_menu();
+    $gui{miDVD}->set_submenu($menu);
+    my $itema = &gui_new_menu_item_no_mnemonic('Add Optical Disc Image', 'cd_add_16px.png', $menu);
+    $itema->signal_connect('activate' => \&show_remote_filechooser, {title => 'Choose Disc Image', entrywidget => '', mode => 'file', filter => '^.*\.iso$', callback => \&devices_menu_add_optical_disc});
+    my $itemb = &gui_new_menu_item_no_mnemonic('Eject Optical Disc', 'cd_clear_16px.png', $menu);
+    $itemb->signal_connect('activate' => \&insert_removable_disk_first_drive, {IMedium => '', type => 'DVD'});
+    $menu->append(&gui_new_menu_sep());
 
     if ($$vhost{dvd}) {
+        my $added;
+
         foreach my $pdvd (@{$$vhost{dvd}}) {
-            my $item = Gtk3::MenuItem->new_with_label('<Server Drive> ' . IMedium_getLocation($pdvd));
-            $dvdmenu->append($item);
-            $item->show();
-            $item->signal_connect(activate => \&mount_dvd_online, $pdvd);
+            $added = 1;
+            my $item = &gui_new_menu_item_no_mnemonic('<Server Drive> ' . IMedium_getLocation($pdvd), 'cd_16px.png', $menu);
+            $item->signal_connect('activate' => \&insert_removable_disk_first_drive, {IMedium => $pdvd, type => 'DVD'});
         }
 
-        my $pdvdsep = Gtk3::SeparatorMenuItem->new();
-        $dvdmenu->append($pdvdsep);
-        $pdvdsep->show();
+        $menu->append(&gui_new_menu_sep()) if ($added); # Only add separator if a host drive was added
     }
 
+    my $IMediumRef = &get_all_media('DVD');
+
     foreach (sort { lc($$IMediumRef{$a}) cmp lc($$IMediumRef{$b}) } (keys %$IMediumRef)) {
-        my $item = Gtk3::MenuItem->new_with_label($$IMediumRef{$_});
-        $dvdmenu->append($item);
-        $item->show();
-        $item->signal_connect(activate => \&mount_dvd_online, $_);
+        my $item = &gui_new_menu_item_no_mnemonic($$IMediumRef{$_}, 'cd_16px.png', $menu);
+        $item->signal_connect('activate' => \&insert_removable_disk_first_drive, {IMedium => $_, type => 'DVD'});
     }
 }
 
@@ -741,9 +732,8 @@ sub fill_menu_usb {
     my $sref = &get_session($$gref{IMachine});
 
     if ($$sref{Lock} eq 'Shared') {
-        my $usbmenu = Gtk3::Menu->new();
-        $gui{menuitemUSB}->set_submenu(undef); # Help garbage collection
-        $gui{menuitemUSB}->set_submenu($usbmenu); # Hijack the temporary submenu (restored on exit)
+        my $usbmenu =  &gui_new_menu();
+        $gui{miUSB}->set_submenu($usbmenu);
         my @IHostUSBDevices = IHost_getUSBDevices(IVirtualBox_getHost($gui{websn}));
         my @USBDevices = IConsole_getUSBDevices(ISession_getConsole($$sref{ISession}));
 
@@ -754,11 +744,9 @@ sub fill_menu_usb {
                                         IUSBDevice_getProduct($usb),
                                         sprintf('%04x', IUSBDevice_getRevision($usb)));
 
-            my $item = Gtk3::CheckMenuItem->new_with_label($label);
+            my $item = &gui_new_check_menu_item($label, $usbmenu);
             my $usbid = IUSBDevice_getId($usb);
             $item->set_active(1) if $connected{$usbid};
-            $usbmenu->append($item);
-            $item->show();
             $item->signal_connect(activate => \&mount_usb_online, [$usbid, $label]);
         }
     }
@@ -772,24 +760,19 @@ sub fill_menu_cpu {
     my $sref = &get_session($$gref{IMachine});
 
     if ($$sref{Lock} eq 'Shared') {
-        my $cpumenu = Gtk3::Menu->new();
-        $gui{menuitemHotPlugCPU}->set_submenu(undef); # Help garbage collection
-        $gui{menuitemHotPlugCPU}->set_submenu($cpumenu); # Hijack the temporary submenu (restored on exit)
+        my $cpumenu = &gui_new_menu();
+        $gui{miHotPlugCPU}->set_submenu($cpumenu);
         my $cpucount = IMachine_getCPUCount($$sref{IMachine});
 
         # CPU 0 is special - it can never be detached.
-        my $item = Gtk3::CheckMenuItem->new_with_label('vCPU 0');
+        my $item = &gui_new_check_menu_item('vCPU 0', $cpumenu);
         $item->set_active(1);
         $item->set_sensitive(0);
         $item->set_tooltip_text('vCPU 0 is never hot pluggable');
-        $cpumenu->append($item);
-        $item->show();
 
         foreach my $cpunum (1..($cpucount - 1)) {
-            my $item = Gtk3::CheckMenuItem->new_with_label("vCPU $cpunum");
+            my $item = &gui_new_check_menu_item("vCPU $cpunum", $cpumenu);
             $item->set_active(1) if (&bl(IMachine_getCPUStatus($$sref{IMachine}, $cpunum)));
-            $cpumenu->append($item);
-            $item->show();
             $item->signal_connect(activate => \&mount_cpu_online, $cpunum);
         }
     }
@@ -844,79 +827,75 @@ sub mount_usb_online {
     ISession_unlockMachine($$sref{ISession}) if (ISession_getState($$sref{ISession}) eq 'Locked');
 }
 
-# Called when parent Floppy menu is highlighted. Cheaper than calling each time machine menu is opened
+# Dynamically fills the floppy menu of the devices menu
 sub fill_menu_floppy {
+    my $gref = &getsel_list_guest();
     my $vhost = &vhost();
-    my $floppymenu = Gtk3::Menu->new();
-    $gui{menuitemFloppy}->set_submenu(undef); # Help garbage collection
-    $gui{menuitemFloppy}->set_submenu($floppymenu); # Hijack the temporary submenu (restored on exit)
-    my $IMediumRef = &get_all_media('Floppy');
-    my $item = Gtk3::MenuItem->new_with_label('<Empty Drive>');
-    $floppymenu->append($item);
-    $item->show();
-    $item->signal_connect(activate => \&mount_floppy_online, '');
-    my $sep = Gtk3::SeparatorMenuItem->new();
-    $floppymenu->append($sep);
-    $sep->show();
+    my $menu = &gui_new_menu();
+    my $itema = &gui_new_menu_item_no_mnemonic('Add Floppy Disk Image', 'fd_add_16px.png', $menu);
+    $itema->signal_connect('activate' => \&show_remote_filechooser, {title => 'Choose Floppy Image', entrywidget => '', mode => 'file', filter => '^.*\.img$', callback => \&devices_menu_add_floppy_disk});
+    my $itemb = &gui_new_menu_item_no_mnemonic('New Floppy Disk Image', 'fd_create_16px.png', $menu);
+    $itemb->signal_connect('activate' => \&show_dialog_create_floppy, {IMachine => $$gref{IMachine}, filename => &rcatfile($$vhost{machinedir}, 'Floppy-' . int(rand(999999))), callback => \&insert_removable_disk_first_drive});
+    my $itemc = &gui_new_menu_item_no_mnemonic('Eject Floppy Disk', 'fd_clear_16px.png', $menu);
+    $itemc->signal_connect('activate' => \&insert_removable_disk_first_drive, {IMedium => '', type => 'Floppy'});
+    $menu->append(&gui_new_menu_sep());
 
     if ($$vhost{floppy}) {
+        my $added;
+
         foreach my $pfloppy (@{$$vhost{floppy}}) {
-            my $item = Gtk3::MenuItem->new_with_label('<Server Drive> ' . IMedium_getLocation($pfloppy));
-            $floppymenu->append($item);
-            $item->show();
-            $item->signal_connect(activate => \&mount_floppy_online, $pfloppy);
+            $added = 1;
+            my $item = &gui_new_menu_item_no_mnemonic('<Server Drive> ' . IMedium_getLocation($pfloppy), 'fd_16px.png', $menu);
+            $item->signal_connect('activate' => \&insert_removable_disk_first_drive, {IMedium => $pfloppy, type => 'Floppy'});
         }
 
-        my $pfloppysep = Gtk3::SeparatorMenuItem->new();
-        $floppymenu->append($pfloppysep);
-        $pfloppysep->show();
+        $menu->append(&gui_new_menu_sep()) if ($added); # Only add separator if a host drive was added
     }
 
+    my $IMediumRef = &get_all_media('Floppy');
+
     foreach (sort { lc($$IMediumRef{$a}) cmp lc($$IMediumRef{$b}) } (keys %$IMediumRef)) {
-        my $item = Gtk3::MenuItem->new_with_label($$IMediumRef{$_});
-        $floppymenu->append($item);
-        $item->show();
-        $item->signal_connect(activate => \&mount_floppy_online, $_);
+        my $item = &gui_new_menu_item_no_mnemonic($$IMediumRef{$_}, 'fd_16px.png', $menu);
+        $item->signal_connect('activate' => \&insert_removable_disk_first_drive, {IMedium => $_, type => 'Floppy'});
     }
+
+    $gui{miFloppy}->set_submenu($menu);
 }
 
-# Inserts a DVD/CD whilst the guest is online and running
-sub mount_dvd_online {
-    my ($widget, $IMedium) = @_;
-    my $gref = &getsel_list_guest();
-    my $sref = &get_session($$gref{IMachine});
-
-    if ($$sref{Lock} eq 'Shared') {
-        my @IMediumAttachment = IMachine_getMediumAttachments($$sref{IMachine});
-        foreach my $attach (@IMediumAttachment) {
-            next if ($$attach{type} ne 'DVD');
-            IMachine_mountMedium($$sref{IMachine}, $$attach{controller}, $$attach{port}, $$attach{device}, $IMedium, 0);
-            last;
-        }
-        IMachine_saveSettings($$sref{IMachine});
-        &addrow_msg_log("Changed optical medium for $$gref{Name}");
-    }
-
-    ISession_unlockMachine($$sref{ISession}) if (ISession_getState($$sref{ISession}) eq 'Locked');
+# Called from the devices menu
+sub devices_menu_add_floppy_disk {
+    my ($basedir, $file) = @_;
+    my $IMedium = &register_medium(undef, {type => 'Floppy', mode => 'ReadWrite', basedir => $basedir, file => $file});
+    &insert_removable_disk_first_drive(undef, {IMedium => $IMedium, type => 'Floppy'}) if ($IMedium);
 }
 
-# Inserts a floppy disk image whilst the guest is online
-sub mount_floppy_online {
-    my ($widget, $IMedium) = @_;
+# Called from the devices menu
+sub devices_menu_add_optical_disc {
+    my ($basedir, $file) = @_;
+    my $IMedium = &register_medium(undef, {type => 'DVD', mode => 'ReadOnly', basedir => $basedir, file => $file});
+    &insert_removable_disk_first_drive(undef, {IMedium => $IMedium, type => 'DVD'}) if ($IMedium);
+}
+
+# Inserts a removable in the first drive found
+sub insert_removable_disk_first_drive {
+    my ($widget, $medium) = @_;
     my $gref = &getsel_list_guest();
     my $sref = &get_session($$gref{IMachine});
+    my @IMediumAttachment = IMachine_getMediumAttachments($$sref{IMachine});
+    my $found = 0;
 
-    if ($$sref{Lock} eq 'Shared') {
-        my @IMediumAttachment = IMachine_getMediumAttachments($$sref{IMachine});
-        foreach my $attach (@IMediumAttachment) {
-            next if ($$attach{type} ne 'Floppy');
-            IMachine_mountMedium($$sref{IMachine}, $$attach{controller}, $$attach{port}, $$attach{device}, $IMedium, 0);
-            last;
-        }
-        IMachine_saveSettings($$sref{IMachine});
-        &addrow_msg_log("Changed floppy medium for $$gref{Name}");
+    foreach my $attach (@IMediumAttachment) {
+        next if ($$attach{type} ne $$medium{type});
+        IMachine_mountMedium($$sref{IMachine}, $$attach{controller}, $$attach{port}, $$attach{device}, $$medium{IMedium}, 0);
+        &onsel_list_guest() if ($gui{treeviewGuest}->get_selection->get_selected());
+        $found = 1;
+        &addrow_msg_log("Changed $$medium{type} medium for $$gref{Name}");
+        last;
     }
 
+    &show_err_msg('nodrive') if ($$medium{type} eq 'DVD' and !$found);
+    &show_err_msg('nofloppydrive') if ($$medium{type} eq 'Floppy' and !$found);
+    IMachine_saveSettings($$sref{IMachine});
     ISession_unlockMachine($$sref{ISession}) if (ISession_getState($$sref{ISession}) eq 'Locked');
 }
 
@@ -977,7 +956,7 @@ sub send_video_hint {
     my $gref = &getsel_list_guest();
     my $sref = &get_session($$gref{IMachine});
 
-    if ($widget eq $gui{menuitemSetVideoCustom}) { $res = &show_dialog_videohint(); }
+    if ($widget eq $gui{miSetVideoCustom}) { $res = &show_dialog_videohint(); }
 
     if ($$sref{IMachine}) {
         my $IConsole = ISession_getConsole($$sref{ISession});
@@ -1066,13 +1045,13 @@ sub imedium_has_property {
 # Set sensitivity based on connection state
 sub sens_connect {
     my ($state) = @_;
-    $gui{menuitemNew}->set_sensitive($state);
-    $gui{menuitemAdd}->set_sensitive($state);
-    $gui{menuitemImportAppl}->set_sensitive($state);
-    $gui{menuitemVMM}->set_sensitive($state);
-    $gui{menuitemHostNetMan}->set_sensitive($state);
-    $gui{menuitemServerInfo}->set_sensitive($state);
-    $gui{menuitemVBPrefs}->set_sensitive($state);
+    $gui{miNew}->set_sensitive($state);
+    $gui{miAdd}->set_sensitive($state);
+    $gui{miImportAppl}->set_sensitive($state);
+    $gui{miVMM}->set_sensitive($state);
+    $gui{miHostNetMan}->set_sensitive($state);
+    $gui{miServerInfo}->set_sensitive($state);
+    $gui{miVBPrefs}->set_sensitive($state);
     $gui{toolbuttonNew}->set_sensitive($state);
     $gui{toolbuttonRefresh}->set_sensitive($state);
     $gui{progressbarMem}->show();
@@ -1080,27 +1059,27 @@ sub sens_connect {
 
 # Sets the sensitivity when no guest is selected
 sub sens_unselected {
-    $gui{menuitemExportAppl}->set_sensitive(0);
-    $gui{menuitemAction}->set_sensitive(0);
-    $gui{menuitemStart}->set_sensitive(0);
-    $gui{menuitemStop}->set_sensitive(0);
-    $gui{menuitemPause}->set_sensitive(0);
-    $gui{menuitemResume}->set_sensitive(0);
-    $gui{menuitemSettings}->set_sensitive(0);
-    $gui{menuitemClone}->set_sensitive(0);
-    $gui{menuitemSetGroup}->set_sensitive(0);
-    $gui{menuitemUngroup}->set_sensitive(0);
-    $gui{menuitemDiscard}->set_sensitive(0);
-    $gui{menuitemReset}->set_sensitive(0);
-    $gui{menuitemRemove}->set_sensitive(0);
-    $gui{menuitemKeyboard}->set_sensitive(0);
-    $gui{menuitemDisplay}->set_sensitive(0);
-    $gui{menuitemHotPlugCPU}->set_sensitive(0);
-    $gui{menuitemDVD}->set_sensitive(0);
-    $gui{menuitemFloppy}->set_sensitive(0);
-    $gui{menuitemUSB}->set_sensitive(0);
-    $gui{menuitemScreenshot}->set_sensitive(0);
-    $gui{menuitemLogs}->set_sensitive(0);
+    $gui{miExportAppl}->set_sensitive(0);
+    $gui{miAction}->set_sensitive(0);
+    $gui{miStart}->set_sensitive(0);
+    $gui{miStop}->set_sensitive(0);
+    $gui{miPause}->set_sensitive(0);
+    $gui{miResume}->set_sensitive(0);
+    $gui{miSettings}->set_sensitive(0);
+    $gui{miClone}->set_sensitive(0);
+    $gui{miSetGroup}->set_sensitive(0);
+    $gui{miUngroup}->set_sensitive(0);
+    $gui{miDiscard}->set_sensitive(0);
+    $gui{miReset}->set_sensitive(0);
+    $gui{miRemove}->set_sensitive(0);
+    $gui{miKeyboard}->set_sensitive(0);
+    $gui{miDisplay}->set_sensitive(0);
+    $gui{miHotPlugCPU}->set_sensitive(0);
+    $gui{miDVD}->set_sensitive(0);
+    $gui{miFloppy}->set_sensitive(0);
+    $gui{miUSB}->set_sensitive(0);
+    $gui{miScreenshot}->set_sensitive(0);
+    $gui{miLogs}->set_sensitive(0);
     $gui{toolbuttonStart}->set_sensitive(0);
     $gui{toolbuttonStop}->set_sensitive(0);
     $gui{toolbuttonSettings}->set_sensitive(0);
@@ -1215,14 +1194,14 @@ sub show_dialog_importappl {
                 my $IProgress = IAppliance_read($IAppliance, $appliancefile);
 
                 if ($IProgress) {
-                    &show_progress_window($IProgress, 'Reading Appliance', $gui{img}{ProgressReadAppl});
+                    &show_progress_window($IProgress, 'Reading Appliance', $gui{pb}{ProgressReadAppl});
                     IAppliance_interpret($IAppliance);
                     my $warnings = IAppliance_getWarnings($IAppliance);
                     &addrow_msg_log("Import warnings: $warnings") if ($warnings);
                     $IProgress = IAppliance_importMachines($IAppliance, @importopts);
 
                     if ($IProgress) {
-                        &show_progress_window($IProgress, 'Importing Appliance', $gui{img}{ProgressImport});
+                        &show_progress_window($IProgress, 'Importing Appliance', $gui{pb}{ProgressImport});
                         my @appluuid = IAppliance_getMachines($IAppliance);
                         foreach my $id (@appluuid) {
                             &makesel_list_guest($id); # Make the current selection the new appliance
