@@ -11,11 +11,11 @@ sub show_dialog_new_guest {
     $gui{radiobuttonNewDynamic}->set_sensitive(1);
     $gui{radiobuttonNewFixed}->set_sensitive(1);
     $gui{radiobuttonNewSplit}->set_sensitive(0);
-    $gui{entryNewName}->set_text('NewGuest-' . int(rand(999999)));
+    $gui{entryNewName}->set_text('Guest-' . int(rand(999999)));
     $gui{spinbuttonNewMemory}->set_range($$vhost{minguestram}, $$vhost{memsize});
     $gui{spinbuttonNewNewHDSize}->set_range($$vhost{minhdsizemb}, $$vhost{maxhdsizemb});
-    $gui{comboboxNewOSFam}->signal_handler_block($signal{fam}); # Block to avoid signal emission when changing
-    $gui{comboboxNewOSVer}->signal_handler_block($signal{ver});
+    $gui{comboboxNewOSFam}->signal_handler_block($signal{comboboxNewOSFam_changed}); # Block to avoid signal emission when changing
+    $gui{comboboxNewOSVer}->signal_handler_block($signal{comboboxNewOSVer_changed});
     $gui{liststoreNewOSFam}->clear();
     $gui{liststoreNewOSVer}->clear();
     $gui{liststoreNewChooseHD}->clear();
@@ -44,10 +44,11 @@ sub show_dialog_new_guest {
     }
     else { $gui{radiobuttonNewExistingHD}->set_sensitive(0); };
 
-    $gui{comboboxNewOSFam}->signal_handler_unblock($signal{fam});
-    $gui{comboboxNewOSVer}->signal_handler_unblock($signal{ver});
+    $gui{comboboxNewOSFam}->signal_handler_unblock($signal{comboboxNewOSFam_changed});
+    $gui{comboboxNewOSVer}->signal_handler_unblock($signal{comboboxNewOSVer_changed});
     $gui{comboboxNewOSFam}->signal_emit('changed'); # Force update of other fields based on OS
     $gui{comboboxNewOSVer}->signal_emit('changed'); # Force update of other fields based on OS
+    &fill_list_new_guest_dvd();
 
     do {
         my $response = $gui{dialogNew}->run;
@@ -64,8 +65,8 @@ sub show_dialog_new_guest {
                 if ($IMachine) {
                     IMachine_applyDefaults($IMachine);
                     IMachine_setMemorySize($IMachine, $gui{spinbuttonNewMemory}->get_value_as_int());
-
-                    my $IAudioAdapter = IMachine_getAudioAdapter($IMachine);
+                    my $IAudioSettings = IMachine_getAudioSettings($IMachine);
+                    my $IAudioAdapter = IAudioSettings_getAdapter($IAudioSettings);
                     IAudioAdapter_setAudioDriver($IAudioAdapter, 'Null');
                     my $IVRDEServer = IMachine_getVRDEServer($IMachine);
                     IVRDEServer_setVRDEProperty($IVRDEServer, 'VideoChannel/Quality', 75);
@@ -79,21 +80,21 @@ sub show_dialog_new_guest {
                     IMachine_saveSettings($IMachine);
                     IVirtualBox_registerMachine($gui{websn}, $IMachine);
                     &addrow_msg_log("Created a new guest: $name");
+                    my $sref = &get_session($IMachine);
 
-                    if ($gui{checkbuttonNewNewHD}->get_active() == 1) {
-                        my $sref = &get_session($IMachine);
-
-                        if ($$sref{Type} eq 'WriteLock') {
+                    if ($$sref{Type} eq 'WriteLock') {
+                        # Hard Disk Section
+                        if ($gui{checkbuttonNewNewHD}->get_active() == 1) {
                             my $IMediumHD;
 
                             if ($gui{radiobuttonNewNewHD}->get_active()) {
                                 my %newhd = (diskname   => $name, # Use guest name as basis for disk
-                                             devicetype => 'HardDisk',
-                                             mode       => 'ReadWrite',
-                                             size       => $gui{spinbuttonNewNewHDSize}->get_value_as_int() * 1048576,
-                                             allocation => ['Standard'], # Standard == Dynamic Allocation
-                                             imgformat  => &getsel_combo($gui{comboboxNewNewFormat}, 1),
-                                             location   => &rcatdir($$vhost{machinedir}, $name));
+                                            devicetype => 'HardDisk',
+                                            mode       => 'ReadWrite',
+                                            size       => $gui{spinbuttonNewNewHDSize}->get_value_as_int() * 1048576,
+                                            allocation => ['Standard'], # Standard == Dynamic Allocation
+                                            imgformat  => &getsel_combo($gui{comboboxNewNewFormat}, 1),
+                                            location   => &rcatdir($$vhost{machinedir}, $name));
 
                                 if ($gui{radiobuttonNewFixed}->get_active()) { $newhd{allocation} = ['Fixed']; }
                                 elsif ($gui{radiobuttonNewSplit}->get_active()) { $newhd{allocation} = ['VmdkSplit2G']; }
@@ -114,12 +115,23 @@ sub show_dialog_new_guest {
                                 my %floppyaddress = &get_free_deviceport($$sref{IMachine}, $IStorCtrFloppy);
                                 IMachine_attachDevice($$sref{IMachine}, 'Floppy', $floppyaddress{portnum}, $floppyaddress{devnum}, 'Floppy', '');
                             }
-
-                            IMachine_saveSettings($$sref{IMachine});
                         }
 
-                        ISession_unlockMachine($$sref{ISession}) if (ISession_getState($$sref{ISession}) eq 'Locked');
+                        # ISO Section
+                        my $IMediumDVD = &getsel_combo($gui{comboboxNewISOImage}, 1);
+                        if (defined($IMediumDVD)) {
+                            my @IMediumAttachment = IMachine_getMediumAttachments($$sref{IMachine});
+                            foreach my $attach (@IMediumAttachment) {
+                                next if ($$attach{type} ne 'DVD');
+                                IMachine_mountMedium($$sref{IMachine}, $$attach{controller}, $$attach{port}, $$attach{device}, $IMediumDVD, 0);
+                                last;
+                            }
+                        }
+
+                        IMachine_saveSettings($$sref{IMachine});
                     }
+
+                    ISession_unlockMachine($$sref{ISession}) if (ISession_getState($$sref{ISession}) eq 'Locked');
                     &fill_list_guest();
                 }
                 else { &show_err_msg('createguest', " ($name)"); }
@@ -234,7 +246,7 @@ sub create_new_clone {
         $IProgress = IMachine_cloneTo($$srcgref{IMachine}, $cloneIMachine, $$cloneref{mode}, @{$$cloneref{options}});
     }
 
-    &show_progress_window($IProgress, 'Cloning Guest', $gui{img}{ProgressClone}) if ($IProgress); # MUST NOT USE $cloneIMachine until progress is complete, otherwise it waits
+    &show_progress_window($IProgress, 'Cloning Guest', $gui{pb}{ProgressClone}) if ($IProgress); # MUST NOT USE $cloneIMachine until progress is complete, otherwise it waits
 
     if (IProgress_getCanceled($IProgress) eq 'true') {
         &addrow_msg_log("Cancelled creation of the clone $$cloneref{name}");
@@ -255,7 +267,7 @@ sub create_new_dskimg_clone {
     $imgfile = &add_ext_if_needed($imgfile, $$newref{imgformat});
     my $IMedium = IVirtualBox_createMedium($gui{websn}, $$newref{imgformat}, $imgfile, $$newref{mode}, $$newref{devicetype});
     my $IProgress = IMedium_cloneTo($srcIMedium, $IMedium, $$newref{allocation}, undef);
-    &show_progress_window($IProgress, 'Copying or Cloning Disk Image', $gui{img}{ProgressMediaCreate});
+    &show_progress_window($IProgress, 'Copying or Cloning Disk Image', $gui{pb}{ProgressMediaCreate});
 
     if (IProgress_getCanceled($IProgress) eq 'true') {
         &addrow_msg_log("Cancelled disk copy or cloning: $imgfile");
@@ -283,7 +295,7 @@ sub create_new_dskimg {
     my $IMedium = IVirtualBox_createMedium($gui{websn}, $$newref{imgformat}, $imgfile, $$newref{mode}, $$newref{devicetype});
     # Note: Modified in the vboxService.pm. We expect an array reference in $$newref{allocation}
     my $IProgress = IMedium_createBaseStorage($IMedium, $$newref{size}, @{$$newref{allocation}});
-    &show_progress_window($IProgress, 'Creating Disk Image', $gui{img}{ProgressMediaCreate});
+    &show_progress_window($IProgress, 'Creating Disk Image', $gui{pb}{ProgressMediaCreate});
 
     if (IProgress_getCanceled($IProgress) eq 'true') {
         &addrow_msg_log("Cancelled disk image creation: $imgfile");
@@ -307,8 +319,8 @@ sub newgen_osfam {
     my ($combofam, $combover) = @_;
     my ($osfam, $osver) = &osfamver();
     my $fam = &getsel_combo($combofam, 1);
-    $combofam->signal_handler_block($signal{fam}); # Block to avoid signal emission when changing
-    $combover->signal_handler_block($signal{ver});
+    $combofam->signal_handler_block($signal{comboboxNewOSFam_changed}); # Block to avoid signal emission when changing
+    $combover->signal_handler_block($signal{comboboxNewOSVer_changed});
     $gui{liststoreNewOSVer}->clear();
 
     foreach my $ver (@{ $$osfam{$fam}{verids} })
@@ -319,8 +331,8 @@ sub newgen_osfam {
     }
 
     $combover->set_active(0) if ($combover->get_active() == -1);
-    $combofam->signal_handler_unblock($signal{fam});
-    $combover->signal_handler_unblock($signal{ver});
+    $combofam->signal_handler_unblock($signal{comboboxNewOSFam_changed});
+    $combover->signal_handler_unblock($signal{comboboxNewOSVer_changed});
     $combover->signal_emit('changed'); # Force update of other fields based on OS
 }
 
@@ -328,12 +340,12 @@ sub newgen_osver {
     my ($combover, $combofam) = @_;
     my $osver = &osver();
     my $ver = &getsel_combo($combover, 1);
-    $combofam->signal_handler_block($signal{fam}); # Avoid signal emission when changing
-    $combover->signal_handler_block($signal{ver});
+    $combofam->signal_handler_block($signal{comboboxNewOSFam_changed}); # Avoid signal emission when changing
+    $combover->signal_handler_block($signal{comboboxNewOSVer_changed});
     $gui{spinbuttonNewMemory}->set_value($$osver{$ver}{recommendedRAM});
     $gui{spinbuttonNewNewHDSize}->set_value($$osver{$ver}{recommendedHDD} / 1048576);
-    $combofam->signal_handler_unblock($signal{fam});
-    $combover->signal_handler_unblock($signal{ver});
+    $combofam->signal_handler_unblock($signal{comboboxNewOSFam_changed});
+    $combover->signal_handler_unblock($signal{comboboxNewOSVer_changed});
 }
 
 sub newstor_new_exist {
