@@ -3,6 +3,14 @@ use strict;
 use warnings;
 our (%gui);
 
+# Registers a medium with the VMM
+sub register_medium {
+    my ($widget, $data) = @_;
+    my $IMedium = IVirtualBox_openMedium($gui{websn}, &rcatfile($$data{basedir}, $$data{file}), $$data{type}, $$data{mode}, 0);
+    &addrow_msg_log("Registered $$data{type}: $$data{file} with VMM");
+    return $IMedium;
+}
+
 sub show_dialog_vmm {
     &addrow_msg_log('Retrieving global media information');
     &busy_window($gui{windowMain}, 0, 'watch');
@@ -21,36 +29,36 @@ sub vmm_tabchanged {
     &vmm_sens_unselected(); # Reset widgets as they are evaluated later
 
     if ($page == 0) { # We need to re-evaluate selection on tab change
-        $gui{toolbuttonVMMAdd}->set_icon_widget($gui{imageHDAdd32});
-        $gui{toolbuttonVMMCreate}->set_icon_widget($gui{imageHDCreate32});
+        $gui{toolbuttonVMMAdd}->set_icon_widget($gui{imgHDAdd32});
+        $gui{toolbuttonVMMCreate}->set_icon_widget($gui{imgHDCreate32});
         $gui{toolbuttonVMMCreate}->set_sensitive(1);
-        $gui{toolbuttonVMMCopy}->set_icon_widget($gui{imageHDCopy32});
-        $gui{toolbuttonVMMMove}->set_icon_widget($gui{imageHDMove32});
-        $gui{toolbuttonVMMModify}->set_icon_widget($gui{imageHDModify32});
-        $gui{toolbuttonVMMRemove}->set_icon_widget($gui{imageHDRemove32});
-        $gui{toolbuttonVMMRelease}->set_icon_widget($gui{imageHDRelease32});
+        $gui{toolbuttonVMMCopy}->set_icon_widget($gui{imgHDCopy32});
+        $gui{toolbuttonVMMMove}->set_icon_widget($gui{imgHDMove32});
+        $gui{toolbuttonVMMModify}->set_icon_widget($gui{imgHDModify32});
+        $gui{toolbuttonVMMRemove}->set_icon_widget($gui{imgHDRemove32});
+        $gui{toolbuttonVMMRelease}->set_icon_widget($gui{imgHDRelease32});
         &onsel_list_vmmhd() if ($gui{treeviewVMMHD}->get_selection->get_selected());
     }
     elsif ($page == 1) {
-        $gui{toolbuttonVMMAdd}->set_icon_widget($gui{imageCDAdd32});
-        $gui{toolbuttonVMMCreate}->set_icon_widget($gui{imageCDCreate32});
+        $gui{toolbuttonVMMAdd}->set_icon_widget($gui{imgDVDAdd32});
+        $gui{toolbuttonVMMCreate}->set_icon_widget($gui{imgDVDCreate32});
         $gui{toolbuttonVMMCreate}->set_sensitive(0);
-        $gui{toolbuttonVMMCopy}->set_icon_widget($gui{imageCDCopy32});
-        $gui{toolbuttonVMMMove}->set_icon_widget($gui{imageCDMove32});
-        $gui{toolbuttonVMMModify}->set_icon_widget($gui{imageCDModify32});
-        $gui{toolbuttonVMMRemove}->set_icon_widget($gui{imageCDRemove32});
-        $gui{toolbuttonVMMRelease}->set_icon_widget($gui{imageCDRelease32});
+        $gui{toolbuttonVMMCopy}->set_icon_widget($gui{imgDVDCopy32});
+        $gui{toolbuttonVMMMove}->set_icon_widget($gui{imgDVDMove32});
+        $gui{toolbuttonVMMModify}->set_icon_widget($gui{imgDVDModify32});
+        $gui{toolbuttonVMMRemove}->set_icon_widget($gui{imgDVDRemove32});
+        $gui{toolbuttonVMMRelease}->set_icon_widget($gui{imgDVDRelease32});
         &onsel_list_vmmdvd() if ($gui{treeviewVMMDVD}->get_selection->get_selected());
     }
     else {
-        $gui{toolbuttonVMMAdd}->set_icon_widget($gui{imageFloppyAdd32});
-        $gui{toolbuttonVMMCreate}->set_icon_widget($gui{imageFloppyCreate32});
+        $gui{toolbuttonVMMAdd}->set_icon_widget($gui{imgFloppyAdd32});
+        $gui{toolbuttonVMMCreate}->set_icon_widget($gui{imgFloppyCreate32});
         $gui{toolbuttonVMMCreate}->set_sensitive(1);
-        $gui{toolbuttonVMMCopy}->set_icon_widget($gui{imageFloppyCopy32});
-        $gui{toolbuttonVMMMove}->set_icon_widget($gui{imageFloppyMove32});
-        $gui{toolbuttonVMMModify}->set_icon_widget($gui{imageFloppyModify32});
-        $gui{toolbuttonVMMRemove}->set_icon_widget($gui{imageFloppyRemove32});
-        $gui{toolbuttonVMMRelease}->set_icon_widget($gui{imageFloppyRelease32});
+        $gui{toolbuttonVMMCopy}->set_icon_widget($gui{imgFloppyCopy32});
+        $gui{toolbuttonVMMMove}->set_icon_widget($gui{imgFloppyMove32});
+        $gui{toolbuttonVMMModify}->set_icon_widget($gui{imgFloppyModify32});
+        $gui{toolbuttonVMMRemove}->set_icon_widget($gui{imgFloppyRemove32});
+        $gui{toolbuttonVMMRelease}->set_icon_widget($gui{imgFloppyRelease32});
         &onsel_list_vmmfloppy() if ($gui{treeviewVMMFloppy}->get_selection->get_selected());
     }
 }
@@ -112,7 +120,7 @@ sub vmm_rem {
 
             if ($MediumState eq 'Created' or $MediumState eq 'Inaccessible') {
                 my $IProgress = IMedium_deleteStorage($$hdref{IMedium});
-                &show_progress_window($IProgress, 'Deleting hard disk', $gui{img}{ProgressMediaDelete});
+                &show_progress_window($IProgress, 'Deleting hard disk', $gui{pb}{ProgressMediaDelete});
                 &addrow_msg_log("Deleted hard disk $$hdref{Name}.");
                 &fill_list_vmmhd();
             }
@@ -144,7 +152,7 @@ sub vmm_compact {
     my $MediumState = IMedium_refreshState($$hdref{IMedium});
     if ($MediumState eq 'Created') {
         my $IProgress = IMedium_compact($$hdref{IMedium});
-        &show_progress_window($IProgress, 'Compacting hard disk', $gui{img}{ProgressMediaResize}) if ($IProgress);
+        &show_progress_window($IProgress, 'Compacting hard disk', $gui{pb}{ProgressMediaResize}) if ($IProgress);
         &addrow_msg_log("Compacted disk image $$hdref{Name}");
         &fill_list_vmmhd();
     }
@@ -172,16 +180,15 @@ sub vmm_move_broker {
 
 # Adds a harddisk/dvd/floppy image to the VMM
 sub vmm_add {
-    my ($location, $filearrayref) = @_;
+    my ($basedir, $filearrayref) = @_;
     my $page = $gui{notebookVMM}->get_current_page();
-    return if (!$location);
+    return if (!$basedir);
 
     if ($page == 0) {
             foreach my $hd (@{$filearrayref}) {
                 next if (!$hd->{FileName});
                 next if ($hd->{FileName} eq '..' or $hd->{Type} eq '(Dir)');
-                IVirtualBox_openMedium($gui{websn}, &rcatfile($location, $hd->{FileName}), 'HardDisk', 'ReadWrite', 0);
-                &addrow_msg_log("Adding Hard Disk $hd->{FileName} to VMM");
+                &register_medium(undef, {type => 'HardDisk', mode => 'ReadWrite', basedir => $basedir, file => $hd->{FileName}});
             }
 
             &fill_list_vmmhd();
@@ -190,8 +197,7 @@ sub vmm_add {
             foreach my $dvd (@{$filearrayref}) {
                 next if (!$dvd->{FileName});
                 next if ($dvd->{FileName} eq '..' or $dvd->{Type} eq '(Dir)');
-                IVirtualBox_openMedium($gui{websn}, &rcatfile($location, $dvd->{FileName}), 'DVD', 'ReadOnly', 0);
-                &addrow_msg_log("Adding DVD/CD $dvd->{FileName} to VMM");
+                &register_medium(undef, {type => 'DVD', mode => 'ReadOnly', basedir => $basedir, file => $dvd->{FileName}});
             }
 
             &fill_list_vmmdvd();
@@ -200,8 +206,7 @@ sub vmm_add {
             foreach my $floppy (@{$filearrayref}) {
                 next if (!$floppy->{FileName});
                 next if ($floppy->{FileName} eq '..' or $floppy->{Type} eq '(Dir)');
-                IVirtualBox_openMedium($gui{websn}, &rcatfile($location, $floppy->{FileName}), 'Floppy', 'ReadWrite', 0);
-                &addrow_msg_log("Adding Floppy $floppy->{FileName} to VMM");
+                &register_medium(undef, {type => 'Floppy', mode => 'ReadWrite', basedir => $basedir, file => $floppy->{FileName}});
             }
 
             &fill_list_vmmfloppy();
@@ -220,10 +225,11 @@ sub vmm_refresh_broker {
 # Selects the type of media to create
 sub vmm_create_broker {
     my $page = $gui{notebookVMM}->get_current_page();
+    my $vhost = &vhost();
 
-    if ($page == 0) { &show_dialog_createhd('VMM'); }
+    if ($page == 0) { &show_dialog_create_hd(undef, {IMachine => '', filename => &rcatfile($$vhost{machinedir}, 'HardDisk-' . int(rand(999999))), callback => \&vmm_refresh_broker}); }
     elsif ($page == 1) { } # Not implemented yet }
-    else { &show_dialog_createfloppy('VMM'); }
+    else { &show_dialog_create_floppy(undef, {IMachine => '', filename => &rcatfile($$vhost{machinedir}, 'Floppy-' . int(rand(999999))), callback => \&vmm_refresh_broker}); }
 }
 
 # Selects the type of media to copy and convert
@@ -247,7 +253,7 @@ sub show_dialog_vmm_move {
 
     if ($response eq 'ok') {
         my $IProgress = IMedium_moveTo($$mediumref{IMedium}, $gui{entryMoveMediumTo}->get_text());
-        &show_progress_window($IProgress, 'Moving medium', $gui{img}{ProgressMediaMove}) if ($IProgress); # Note: An $IProgress is not always returned for move operations are some are very quick
+        &show_progress_window($IProgress, 'Moving medium', $gui{pb}{ProgressMediaMove}) if ($IProgress); # Note: An $IProgress is not always returned for move operations are some are very quick
         &addrow_msg_log('Moved medium ' . $$mediumref{Location} . ' to ' . $gui{entryMoveMediumTo}->get_text());
 
         if ($devicetype eq 'HardDisk') { &fill_list_vmmhd(); }
@@ -274,7 +280,7 @@ sub show_dialog_vmm_modify {
         if ($newhdsizemb > $hdsizemb and $gui{checkbuttonVMMModifyResizeHD}->get_active()) {
             &addrow_msg_log("Resizing $$hdref{Name} ($hdsizemb MB -> $newhdsizemb MB)");
             my $IProgress = IMedium_resize($$hdref{IMedium}, ($newhdsizemb * 1048576));
-            &show_progress_window($IProgress, 'Resizing Virtual Hard Disk Image', $gui{img}{ProgressMediaResize}) if ($IProgress);
+            &show_progress_window($IProgress, 'Resizing Virtual Hard Disk Image', $gui{pb}{ProgressMediaResize}) if ($IProgress);
             my $MediumState = IMedium_refreshState($$hdref{IMedium}); # Make sure VB sees any size changes
             my $refreshsize = IMedium_getLogicalSize($$hdref{IMedium});
             &show_err_msg('nohdresize', "\nMedium: $$hdref{Name}\nCurrent State: $MediumState") if (($refreshsize / 1048576) < $newhdsizemb); # See if actually got resized, not all types support resizing
@@ -395,4 +401,85 @@ sub vmm_modify_sens_resize {
     $gui{spinbuttonVMMModifyResizeHD}->set_sensitive($state);
 }
 
+# Displays the create a new floppy disk dialog. Creates & attaches based on options
+# $data should be {IMachine => $IMachine, filename, callback => \&callbacksub}
+sub show_dialog_create_floppy {
+    my ($widget, $data) = @_;
+    my $callback = $$data{callback};
+    $gui{comboboxCreateFloppySize}->set_active(1);
+    $gui{entryCreateFloppyName}->set_text($$data{filename});
+
+    do {
+        my $response = $gui{dialogCreateFloppy}->run;
+
+        if ($response eq 'ok') {
+            # No validation needed on other entries
+            if (!$gui{entryCreateFloppyName}->get_text()) { &show_err_msg('invalidname'); }
+            else {
+                $gui{dialogCreateFloppy}->hide;
+                my ($vol, $dir, $file) = &rsplitpath($gui{entryCreateFloppyName}->get_text());
+                my $alloc = ['Fixed'];
+                $alloc = ['Fixed', 'Formatted'] if ($gui{checkbuttonCreateFloppyFAT12}->get_active());
+
+                my %newfloppy = (diskname   => $file,
+                                 devicetype => 'Floppy',
+                                 mode       => 'ReadWrite',
+                                 size       => &getsel_combo($gui{comboboxCreateFloppySize}, 1),
+                                 allocation => $alloc,
+                                 imgformat  => 'raw',
+                                 location   => $vol . $dir);
+
+                my $IMedium = &create_new_dskimg(\%newfloppy);
+                # Execute the callback. First paramater is calling widget
+                &$callback($gui{dialogCreateFloppy}, {IMedium => $IMedium, type => 'Floppy'}) if ($callback);
+            }
+        }
+        else { $gui{dialogCreateFloppy}->hide; }
+
+    } until (!$gui{dialogCreateFloppy}->get_visible());
+}
+
+# Displays the create a new hard disk dialog. Creates & attaches based on options
+sub show_dialog_create_hd {
+    my ($widget, $data) = @_;
+    my $vhost = &vhost();
+    $gui{entryCreateHDName}->set_text($$data{filename});
+    $gui{comboboxCreateHDFormat}->set_active(0);
+    $gui{radiobuttonCreateHDDynamic}->show();
+    $gui{radiobuttonCreateHDFixed}->show();
+    $gui{radiobuttonCreateHDSplit}->hide();
+    $gui{spinbuttonCreateHDSize}->set_range($$vhost{minhdsizemb}, $$vhost{maxhdsizemb});
+    $gui{spinbuttonCreateHDSize}->set_value(8192.00);
+
+    do {
+        my $response = $gui{dialogCreateHD}->run;
+
+        if ($response eq 'ok') {
+            # No validation needed on other entries
+            if (!$gui{entryCreateHDName}->get_text()) { &show_err_msg('invalidname'); }
+            else {
+                $gui{dialogCreateHD}->hide;
+                my ($vol, $dir, $file) = &rsplitpath($gui{entryCreateHDName}->get_text());
+
+                my %newhd = (diskname   => $file,
+                             devicetype => 'HardDisk',
+                             mode       => 'ReadWrite',
+                             size       => $gui{spinbuttonCreateHDSize}->get_value_as_int() * 1048576,
+                             allocation => ['Standard'], # Standard == Dynamic Allocation
+                             imgformat  => &getsel_combo($gui{comboboxCreateHDFormat}, 1),
+                             location   => $vol . $dir);
+
+                if ($gui{radiobuttonCreateHDFixed}->get_active()) { $newhd{allocation} = ['Fixed']; }
+                elsif ($gui{radiobuttonCreateHDSplit}->get_active()) { $newhd{allocation} = ['VmdkSplit2G']; }
+                my $IMedium = &create_new_dskimg(\%newhd);
+
+                # Execute the callback. First paramater is calling widget
+                &{$$data{callback}}($gui{dialogCreateHD}, {IMedium => $IMedium, type => 'HardDisk'}) if ($$data{callback});
+            }
+        }
+        else { $gui{dialogCreateHD}->hide; }
+
+    } until (!$gui{dialogCreateHD}->get_visible());
+}
+
 1;
