@@ -2,6 +2,12 @@
 use strict;
 use warnings;
 
+sub SOAP::Serializer::as_APICMode {
+    my $self = shift;
+    my ($value, $name, $type, $attr) = @_;
+    return [$name, {'xsi:type' => 'vbox:APICMode', %$attr}, $value];
+}
+
 sub SOAP::Serializer::as_AccessMode {
     my $self = shift;
     my ($value, $name, $type, $attr) = @_;
@@ -38,6 +44,24 @@ sub SOAP::Serializer::as_AudioControllerType {
     return [$name, {'xsi:type' => 'vbox:AudioControllerType', %$attr}, $value];
 }
 
+sub SOAP::Serializer::as_AudioDeviceState {
+    my $self = shift;
+    my ($value, $name, $type, $attr) = @_;
+    return [$name, {'xsi:type' => 'vbox:AudioDeviceState', %$attr}, $value];
+}
+
+sub SOAP::Serializer::as_AudioDeviceType {
+    my $self = shift;
+    my ($value, $name, $type, $attr) = @_;
+    return [$name, {'xsi:type' => 'vbox:AudioDeviceType', %$attr}, $value];
+}
+
+sub SOAP::Serializer::as_AudioDirection {
+    my $self = shift;
+    my ($value, $name, $type, $attr) = @_;
+    return [$name, {'xsi:type' => 'vbox:AudioDirection', %$attr}, $value];
+}
+
 sub SOAP::Serializer::as_AudioDriverType {
     my $self = shift;
     my ($value, $name, $type, $attr) = @_;
@@ -74,6 +98,12 @@ sub SOAP::Serializer::as_BitmapFormat {
     return [$name, {'xsi:type' => 'vbox:BitmapFormat', %$attr}, $value];
 }
 
+sub SOAP::Serializer::as_CPUArchitecture {
+    my $self = shift;
+    my ($value, $name, $type, $attr) = @_;
+    return [$name, {'xsi:type' => 'vbox:CPUArchitecture', %$attr}, $value];
+}
+
 sub SOAP::Serializer::as_CPUPropertyType {
     my $self = shift;
     my ($value, $name, $type, $attr) = @_;
@@ -110,22 +140,52 @@ sub SOAP::Serializer::as_CloneOptions {
     return [$name, {'xsi:type' => 'vbox:CloneOptions', %$attr}, $value];
 }
 
+sub SOAP::Serializer::as_CloudImageState {
+    my $self = shift;
+    my ($value, $name, $type, $attr) = @_;
+    return [$name, {'xsi:type' => 'vbox:CloudImageState', %$attr}, $value];
+}
+
+sub SOAP::Serializer::as_CloudMachineState {
+    my $self = shift;
+    my ($value, $name, $type, $attr) = @_;
+    return [$name, {'xsi:type' => 'vbox:CloudMachineState', %$attr}, $value];
+}
+
+sub SOAP::Serializer::as_DHCPConfigScope {
+    my $self = shift;
+    my ($value, $name, $type, $attr) = @_;
+    return [$name, {'xsi:type' => 'vbox:DHCPConfigScope', %$attr}, $value];
+}
+
+sub SOAP::Serializer::as_DHCPGroupConditionType {
+    my $self = shift;
+    my ($value, $name, $type, $attr) = @_;
+    return [$name, {'xsi:type' => 'vbox:DHCPGroupConditionType', %$attr}, $value];
+}
+
+sub SOAP::Serializer::as_DHCPOption {
+    my $self = shift;
+    my ($value, $name, $type, $attr) = @_;
+    return [$name, {'xsi:type' => 'vbox:DHCPOption', %$attr}, $value];
+}
+
+sub SOAP::Serializer::as_DHCPOptionEncoding {
+    my $self = shift;
+    my ($value, $name, $type, $attr) = @_;
+    return [$name, {'xsi:type' => 'vbox:DHCPOptionEncoding', %$attr}, $value];
+}
+
 sub SOAP::Serializer::as_DeviceType {
     my $self = shift;
     my ($value, $name, $type, $attr) = @_;
     return [$name, {'xsi:type' => 'vbox:DeviceType', %$attr}, $value];
 }
 
-sub SOAP::Serializer::as_DhcpOpt {
+sub SOAP::Serializer::as_DirectoryCopyFlag {
     my $self = shift;
     my ($value, $name, $type, $attr) = @_;
-    return [$name, {'xsi:type' => 'vbox:DhcpOpt', %$attr}, $value];
-}
-
-sub SOAP::Serializer::as_DirectoryCopyFlags {
-    my $self = shift;
-    my ($value, $name, $type, $attr) = @_;
-    return [$name, {'xsi:type' => 'vbox:DirectoryCopyFlags', %$attr}, $value];
+    return [$name, {'xsi:type' => 'vbox:DirectoryCopyFlag', %$attr}, $value];
 }
 
 sub SOAP::Serializer::as_DirectoryCreateFlag {
@@ -164,12 +224,6 @@ sub SOAP::Serializer::as_ExportOptions {
     return [$name, {'xsi:type' => 'vbox:ExportOptions', %$attr}, $value];
 }
 
-sub SOAP::Serializer::as_FaultToleranceState {
-    my $self = shift;
-    my ($value, $name, $type, $attr) = @_;
-    return [$name, {'xsi:type' => 'vbox:FaultToleranceState', %$attr}, $value];
-}
-
 sub SOAP::Serializer::as_FileAccessMode {
     my $self = shift;
     my ($value, $name, $type, $attr) = @_;
@@ -188,10 +242,10 @@ sub SOAP::Serializer::as_FileOpenAction {
     return [$name, {'xsi:type' => 'vbox:FileOpenAction', %$attr}, $value];
 }
 
-sub SOAP::Serializer::as_FileOpenExFlags {
+sub SOAP::Serializer::as_FileOpenExFlag {
     my $self = shift;
     my ($value, $name, $type, $attr) = @_;
-    return [$name, {'xsi:type' => 'vbox:FileOpenExFlags', %$attr}, $value];
+    return [$name, {'xsi:type' => 'vbox:FileOpenExFlag', %$attr}, $value];
 }
 
 sub SOAP::Serializer::as_FileSeekOrigin {
@@ -212,10 +266,10 @@ sub SOAP::Serializer::as_FirmwareType {
     return [$name, {'xsi:type' => 'vbox:FirmwareType', %$attr}, $value];
 }
 
-sub SOAP::Serializer::as_FsObjMoveFlags {
+sub SOAP::Serializer::as_FsObjMoveFlag {
     my $self = shift;
     my ($value, $name, $type, $attr) = @_;
-    return [$name, {'xsi:type' => 'vbox:FsObjMoveFlags', %$attr}, $value];
+    return [$name, {'xsi:type' => 'vbox:FsObjMoveFlag', %$attr}, $value];
 }
 
 sub SOAP::Serializer::as_FsObjRenameFlag {
@@ -230,12 +284,36 @@ sub SOAP::Serializer::as_GraphicsControllerType {
     return [$name, {'xsi:type' => 'vbox:GraphicsControllerType', %$attr}, $value];
 }
 
+sub SOAP::Serializer::as_GuestDebugIoProvider {
+    my $self = shift;
+    my ($value, $name, $type, $attr) = @_;
+    return [$name, {'xsi:type' => 'vbox:GuestDebugIoProvider', %$attr}, $value];
+}
+
+sub SOAP::Serializer::as_GuestDebugProvider {
+    my $self = shift;
+    my ($value, $name, $type, $attr) = @_;
+    return [$name, {'xsi:type' => 'vbox:GuestDebugProvider', %$attr}, $value];
+}
+
+sub SOAP::Serializer::as_GuestMonitorStatus {
+    my $self = shift;
+    my ($value, $name, $type, $attr) = @_;
+    return [$name, {'xsi:type' => 'vbox:GuestMonitorStatus', %$attr}, $value];
+}
+
 sub SOAP::Serializer::as_GuestSessionWaitForFlag {
     my $self = shift;
     my ($value, $name, $type, $attr) = @_;
     return [$name, {'xsi:type' => 'vbox:GuestSessionWaitForFlag', %$attr}, $value];
 }
 
+sub SOAP::Serializer::as_GuestShutdownFlag {
+    my $self = shift;
+    my ($value, $name, $type, $attr) = @_;
+    return [$name, {'xsi:type' => 'vbox:GuestShutdownFlag', %$attr}, $value];
+}
+
 sub SOAP::Serializer::as_HWVirtExPropertyType {
     my $self = shift;
     my ($value, $name, $type, $attr) = @_;
@@ -254,6 +332,12 @@ sub SOAP::Serializer::as_ImportOptions {
     return [$name, {'xsi:type' => 'vbox:ImportOptions', %$attr}, $value];
 }
 
+sub SOAP::Serializer::as_IommuType {
+    my $self = shift;
+    my ($value, $name, $type, $attr) = @_;
+    return [$name, {'xsi:type' => 'vbox:IommuType', %$attr}, $value];
+}
+
 sub SOAP::Serializer::as_KeyboardHIDType {
     my $self = shift;
     my ($value, $name, $type, $attr) = @_;
@@ -308,6 +392,12 @@ sub SOAP::Serializer::as_ParavirtProvider {
     return [$name, {'xsi:type' => 'vbox:ParavirtProvider', %$attr}, $value];
 }
 
+sub SOAP::Serializer::as_PartitionTableType {
+    my $self = shift;
+    my ($value, $name, $type, $attr) = @_;
+    return [$name, {'xsi:type' => 'vbox:PartitionTableType', %$attr}, $value];
+}
+
 sub SOAP::Serializer::as_PointingHIDType {
     my $self = shift;
     my ($value, $name, $type, $attr) = @_;
@@ -350,6 +440,66 @@ sub SOAP::Serializer::as_ProcessorFeature {
     return [$name, {'xsi:type' => 'vbox:ProcessorFeature', %$attr}, $value];
 }
 
+sub SOAP::Serializer::as_ProxyMode {
+    my $self = shift;
+    my ($value, $name, $type, $attr) = @_;
+    return [$name, {'xsi:type' => 'vbox:ProxyMode', %$attr}, $value];
+}
+
+sub SOAP::Serializer::as_RecordingAudioCodec {
+    my $self = shift;
+    my ($value, $name, $type, $attr) = @_;
+    return [$name, {'xsi:type' => 'vbox:RecordingAudioCodec', %$attr}, $value];
+}
+
+sub SOAP::Serializer::as_RecordingCodecDeadline {
+    my $self = shift;
+    my ($value, $name, $type, $attr) = @_;
+    return [$name, {'xsi:type' => 'vbox:RecordingCodecDeadline', %$attr}, $value];
+}
+
+sub SOAP::Serializer::as_RecordingDestination {
+    my $self = shift;
+    my ($value, $name, $type, $attr) = @_;
+    return [$name, {'xsi:type' => 'vbox:RecordingDestination', %$attr}, $value];
+}
+
+sub SOAP::Serializer::as_RecordingFeature {
+    my $self = shift;
+    my ($value, $name, $type, $attr) = @_;
+    return [$name, {'xsi:type' => 'vbox:RecordingFeature', %$attr}, $value];
+}
+
+sub SOAP::Serializer::as_RecordingRateControlMode {
+    my $self = shift;
+    my ($value, $name, $type, $attr) = @_;
+    return [$name, {'xsi:type' => 'vbox:RecordingRateControlMode', %$attr}, $value];
+}
+
+sub SOAP::Serializer::as_RecordingVideoCodec {
+    my $self = shift;
+    my ($value, $name, $type, $attr) = @_;
+    return [$name, {'xsi:type' => 'vbox:RecordingVideoCodec', %$attr}, $value];
+}
+
+sub SOAP::Serializer::as_RecordingVideoScalingMode {
+    my $self = shift;
+    my ($value, $name, $type, $attr) = @_;
+    return [$name, {'xsi:type' => 'vbox:RecordingVideoScalingMode', %$attr}, $value];
+}
+
+sub SOAP::Serializer::as_ScreenLayoutMode {
+    my $self = shift;
+    my ($value, $name, $type, $attr) = @_;
+    return [$name, {'xsi:type' => 'vbox:ScreenLayoutMode', %$attr}, $value];
+}
+
+sub SOAP::Serializer::as_SignatureType {
+    my $self = shift;
+    my ($value, $name, $type, $attr) = @_;
+    return [$name, {'xsi:type' => 'vbox:SignatureType', %$attr}, $value];
+}
+
 sub SOAP::Serializer::as_StorageBus {
     my $self = shift;
     my ($value, $name, $type, $attr) = @_;
@@ -374,6 +524,12 @@ sub SOAP::Serializer::as_SymlinkType {
     return [$name, {'xsi:type' => 'vbox:SymlinkType', %$attr}, $value];
 }
 
+sub SOAP::Serializer::as_TpmType {
+    my $self = shift;
+    my ($value, $name, $type, $attr) = @_;
+    return [$name, {'xsi:type' => 'vbox:TpmType', %$attr}, $value];
+}
+
 sub SOAP::Serializer::as_USBControllerType {
     my $self = shift;
     my ($value, $name, $type, $attr) = @_;
@@ -386,12 +542,36 @@ sub SOAP::Serializer::as_USBDeviceFilterAction {
     return [$name, {'xsi:type' => 'vbox:USBDeviceFilterAction', %$attr}, $value];
 }
 
+sub SOAP::Serializer::as_UartType {
+    my $self = shift;
+    my ($value, $name, $type, $attr) = @_;
+    return [$name, {'xsi:type' => 'vbox:UartType', %$attr}, $value];
+}
+
+sub SOAP::Serializer::as_UefiVariableAttributes {
+    my $self = shift;
+    my ($value, $name, $type, $attr) = @_;
+    return [$name, {'xsi:type' => 'vbox:UefiVariableAttributes', %$attr}, $value];
+}
+
+sub SOAP::Serializer::as_UpdateChannel {
+    my $self = shift;
+    my ($value, $name, $type, $attr) = @_;
+    return [$name, {'xsi:type' => 'vbox:UpdateChannel', %$attr}, $value];
+}
+
 sub SOAP::Serializer::as_VBoxEventType {
     my $self = shift;
     my ($value, $name, $type, $attr) = @_;
     return [$name, {'xsi:type' => 'vbox:VBoxEventType', %$attr}, $value];
 }
 
+sub SOAP::Serializer::as_VMProcPriority {
+    my $self = shift;
+    my ($value, $name, $type, $attr) = @_;
+    return [$name, {'xsi:type' => 'vbox:VMProcPriority', %$attr}, $value];
+}
+
 sub SOAP::Serializer::as_VirtualSystemDescriptionType {
     my $self = shift;
     my ($value, $name, $type, $attr) = @_;
